package net.wanji.opt.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.opt.dao.mapper.CrossInfoMapper;
import net.wanji.opt.dao.mapper.trend.CrossDataRealtimeMapper;
import net.wanji.opt.dao.mapper.trend.GreenwaveInfoMapper;
import net.wanji.opt.dto.trend.AbnormalCrossListDTO;
import net.wanji.opt.dto.trend.GreenwaveListDTO;
import net.wanji.opt.po.base.CrossInfoPO;
import net.wanji.opt.service.TrendService;
import net.wanji.opt.vo.AbnormalCrossListVO;
import net.wanji.opt.vo.AbnormalCrossStats;
import net.wanji.opt.vo.AbnormalCrossVO;
import net.wanji.opt.vo.GreenwaveListVO;
import net.wanji.opt.vo.GreenwaveStats;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/2/8 10:31
 */
@Service
public class TrendServiceImpl implements TrendService {
    private final GreenwaveInfoMapper greenwaveInfoMapper;
    private final CrossInfoMapper crossInfoMapper;
    private final CrossDataRealtimeMapper crossDataRealtimeMapper;

    public TrendServiceImpl(GreenwaveInfoMapper greenwaveInfoMapper, CrossInfoMapper crossInfoMapper, CrossDataRealtimeMapper crossDataRealtimeMapper) {
        this.greenwaveInfoMapper = greenwaveInfoMapper;
        this.crossInfoMapper = crossInfoMapper;
        this.crossDataRealtimeMapper = crossDataRealtimeMapper;
    }

    @Override
    public List<GreenwaveListVO> greenwaveList(GreenwaveListDTO greenwaveListDTO) {
        Integer status = greenwaveListDTO.getStatus();
        String name = greenwaveListDTO.getName();
        Integer type = greenwaveListDTO.getType();
        List<GreenwaveListVO> greenwaveListVOList = greenwaveInfoMapper.listGreenwave(status, name, type);
        // 3以上都算拥堵
        if (status != null && status == 3) {
            List<GreenwaveListVO> extraList = greenwaveInfoMapper.listGreenwave(4, name, type);
            greenwaveListVOList.addAll(extraList);
        }
        changeStatus(greenwaveListVOList);
        for (GreenwaveListVO greenwaveListVO : greenwaveListVOList) {
            // 计算持续时间
            buildDuration(greenwaveListVO);
            // 构造路口列表
            String greenwaveId = greenwaveListVO.getId();
            List<GreenwaveListVO.CrossListElement> crossList = buildCrossList(greenwaveId);
            greenwaveListVO.setCrossList(crossList);
        }
        return greenwaveListVOList;
    }

    @Override
    public GreenwaveStats buildGreenwaveStats(List<GreenwaveListVO> greenwaveListVOList) {
        GreenwaveStats greenwaveStats = new GreenwaveStats();
        Integer smooth = greenwaveStats.getSmooth();
        Integer slow = greenwaveStats.getSlow();
        Integer congestion = greenwaveStats.getCongestion();
        for (GreenwaveListVO greenwaveListVO : greenwaveListVOList) {
            Integer realtimeStatus = greenwaveListVO.getRealtimeStatus();
            if (realtimeStatus == 1) {
                smooth += 1;
            } else if (realtimeStatus == 2) {
                slow += 1;
            } else if (realtimeStatus == 3) {
                congestion += 1;
            }
        }
        greenwaveStats.setSmooth(smooth);
        greenwaveStats.setSlow(slow);
        greenwaveStats.setCongestion(congestion);
        return greenwaveStats;
    }

    @Override
    public AbnormalCrossVO abnormalCrossList(AbnormalCrossListDTO abnormalCrossListDTO) {
        AbnormalCrossVO abnormalCrossVO = new AbnormalCrossVO();
        // 构建问题路口列表
        Integer status = abnormalCrossListDTO.getStatus();
        String name = abnormalCrossListDTO.getName();
        Integer type = abnormalCrossListDTO.getType();
        List<AbnormalCrossListVO> abnormalCrossListVOList =
                crossDataRealtimeMapper.selectAbnormalCross(status, name, type);
        // 坐标格式转换
        for (AbnormalCrossListVO abnormalCrossListVO : abnormalCrossListVOList) {
            String locationStr = abnormalCrossListVO.getLocationStr();
            double[] lonLat = CrossUtil.getLonLat(locationStr);
            List<Double> location = new ArrayList<>();
            location.add(lonLat[0]);
            location.add(lonLat[1]);
            abnormalCrossListVO.setLocation(location);
        }
        abnormalCrossVO.setAbnormalCrossList(abnormalCrossListVOList);
        // 构造统计信息
        List<AbnormalCrossListVO> listForStats =
                crossDataRealtimeMapper.selectAbnormalCross(null, null, null);
        buildAbnormalCrossStats(abnormalCrossVO, listForStats);

        return abnormalCrossVO;
    }

    private static void buildAbnormalCrossStats(AbnormalCrossVO abnormalCrossVO,
                                                List<AbnormalCrossListVO> abnormalCrossListVOList) {
        AbnormalCrossStats abnormalCrossStats = new AbnormalCrossStats();
        Integer unbalance = abnormalCrossStats.getUnbalance();
        Integer congestion = abnormalCrossStats.getCongestion();
        Integer spillover = abnormalCrossStats.getSpillover();
        for (AbnormalCrossListVO abnormalCrossListVO : abnormalCrossListVOList) {
            Integer realtimeStatus = abnormalCrossListVO.getRealtimeStatus();
            if (realtimeStatus == 1) {
                unbalance += 1;
            } else if (realtimeStatus == 2) {
                congestion += 1;
            } else if (realtimeStatus == 3) {
                spillover += 1;
            }
            if (abnormalCrossListVO.getIsUnbalance() == 1) {
                unbalance += 1;
            }
            if (abnormalCrossListVO.getIsCongestion() ==1) {
                congestion += 1;
            }
            if (abnormalCrossListVO.getIsSpillover() == 1) {
                spillover += 1;
            }
        }
        abnormalCrossStats.setUnbalance(unbalance);
        abnormalCrossStats.setCongestion(congestion);
        abnormalCrossStats.setSpillover(spillover);
        abnormalCrossVO.setAbnormalCrossStats(abnormalCrossStats);
    }

    private static void changeStatus(List<GreenwaveListVO> greenwaveListVOList) {
        if (greenwaveListVOList != null) {
            for (GreenwaveListVO greenwaveListVO : greenwaveListVOList) {
                if (greenwaveListVO.getRealtimeStatus() == 4) {
                    greenwaveListVO.setRealtimeStatus(3);
                }
            }
        }
    }

    private List<GreenwaveListVO.CrossListElement> buildCrossList(String greenwaveId) {
        List<GreenwaveListVO.CrossListElement> crossList = new ArrayList<>();
        List<String> crossIdList = greenwaveInfoMapper.selectCrossIdsById(greenwaveId);
        if (CollectionUtil.isNotEmpty(crossIdList)) {
            List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectByCrossIds(crossIdList);
            for (CrossInfoPO crossInfoPO : crossInfoPOList) {
                GreenwaveListVO.CrossListElement crossListElement = new GreenwaveListVO.CrossListElement();
                crossListElement.setCrossId(crossInfoPO.getId());
                crossListElement.setName(crossInfoPO.getName());
                List<Double> location = buildLocation(crossInfoPO);
                crossListElement.setLocation(location);

                crossList.add(crossListElement);
            }
        }
        return crossList;
    }

    private List<Double> buildLocation(CrossInfoPO crossInfoPO) {
        String location = crossInfoPO.getLocation();
        double[] lonLat = CrossUtil.getLonLat(location);
        List<Double> res = new ArrayList<>();
        res.add(lonLat[0]);
        res.add(lonLat[1]);
        return res;
    }

    private static void buildDuration(GreenwaveListVO greenwaveListVO) {
        Date startTime = greenwaveListVO.getStartTime();
        Date endTime = greenwaveListVO.getEndTime();
        Long duration = null;
        if (endTime != null) {
            duration = DateUtil.between(startTime, endTime, DateUnit.MINUTE);
        } else {
            duration = DateUtil.between(startTime, new Date(), DateUnit.MINUTE);
        }
        greenwaveListVO.setDuration(duration);
    }
}
